#!/bin/sh
#   Copyright (c) 2010 digitalSTROM.org, Zurich, Switzerland
#
#   Author: Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
#
#   This file is part of digitalSTROM Server.
#
#   funclib.sh is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   funclib.sh is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this file. If not, see <http://www.gnu.org/licenses/>.

. ./jsonlib.sh
. ./util.sh

check_in_range() {
    if [[ -z "$1" ]] ; then
        return 1
    fi

    echo $1 | grep "[^0-9]" > /dev/null 2>&1
    if [ "$?" -eq "0" ]; then
        return 1
    fi

    if [[ $1 -lt 0 ]]; then
        return 2
    fi

    if [[ $1 -gt 255 ]]; then
        return 3
    fi

    return 0
}

validate_ip() {
    if [[ -z "$1" ]] ; then
        return 1
    fi

    N1=`echo $1 | cut -d '.' -f 1`

    check_in_range $N1
    if [[ "$?" -ne "0" ]]; then
        return 1
    fi

    N2=`echo $1 | cut -d '.' -f 2`
    check_in_range $N2
    if [[ "$?" -ne "0" ]]; then
        return 1
    fi

    N3=`echo $1 | cut -d '.' -f 3`
    check_in_range $N3
    if [[ "$?" -ne "0" ]]; then
        return 1
    fi
    N4=`echo $1 | cut -d '.' -f 4`
    check_in_range $N4
    if [[ "$?" -ne "0" ]]; then
        return 1
    fi

    if [[ "$N1" -eq "0" ]] && [[ "$N2" -eq "0" ]] &&
       [[ "$N3" -eq "0" ]] && [[ "$N4" -eq "0" ]]; then
        return 1
    fi

    if [[ "$N1" -eq "255" ]] && [[ "$N2" -eq "255" ]] &&
       [[ "$N3" -eq "255" ]] && [[ "$N4" -eq "255" ]]; then
        return 1
    fi

    if [[ "$N1" -eq "0" ]]; then
        return 1
    fi

    if [[ "$N4" -eq "255" ]]; then
        return 1
    fi

    return 0
}

restart_net_with_redirect() {
    ERR=0
    export PATH

    # get current settings
    OLD_SETTINGS=`./netinfo.sh console`

    ifdown eth0 2>&1
    ifconfig eth0 down 2>&1

    ifconfig eth0:1 down 1>&2

    ifup eth0 1>&2
    if [[ "$?" -ne "0" ]]; then
        ERR=$ERR_NETUP
    else
      sleep 1
    fi


    # the survive flag indicates if the old connection must survive, so
    # that the web user interface is able to get data about the new
    # configuration
    SURVIVE=0

    NEW_SETTINGS=`./netinfo.sh console`

    OLD_IP=`echo $OLD_SETTINGS | cut -d ":" -f 2`
    OLD_NETMASK=`echo $OLD_SETTINGS | cut -d ":" -f 3`

    NEW_IP=`echo $NEW_SETTINGS | cut -d ":" -f 2`
    NEW_NETMASK=`echo $NEW_SETTINGS | cut -d ":" -f 3`

    if [[ -n "$OLD_IP" ]] && [[ -n "$OLD_NETMASK" ]]; then
        if [[ "$OLD_IP" != "$NEW_IP" ]] ||
            [[ "$OLD_NETMASK" != "$NEW_NETMASK" ]]; then
            SURVIVE=1
        fi
    fi

    echo "$ERR:$SURVIVE:$OLD_IP:$OLD_NETMASK:$NEW_IP:$NEW_NETMASK"
    return $ERR
}

redirect() {
    OLD_IP=`echo $1 | cut -d ":" -f 3`
    OLD_NETMASK=`echo $1 | cut -d ":" -f 4`
    NEW_IP=`echo $1 | cut -d ":" -f 5`
    NEW_NETMASK=`echo $1 | cut -d ":" -f 6`

    # if we fail to bring up the old ip there is actually no point in
    # printing anything further - the browser will not get it anymore
    ifconfig eth0:1 down 1>&2
    ifconfig eth0:1 "$OLD_IP" netmask "$OLD_NETMASK" 1>&2
    RETCODE="$?"
    if [[ "$RETCODE" -eq "0" ]]; then
        echo "$NEW_IP:$NEW_NETMASK"
        return $ERR_REDIRECT
    else
        log "Failed to configure backup interface"
    fi

    return $ERR_TEMPFAIL
}

down_backup_if() {
    if [[ -x /usr/bin/at ]]; then
        at now + 1 minute < /www/scripts/bkifdown.sh > /dev/null 2>&1
    else
        sleep 3
        ifconfig eth0:1 down 1>&2
    fi
}

get_current_ip() {
    IP=`ifconfig eth0 | grep 'inet addr:' | cut -d: -f2 | cut -d " " -f1`
    if [[ -z "$IP" ]]; then
        IP=`ifconfig eth0:avahi | grep 'inet addr:' | cut -d: -f2 | cut -d " " -f1`
    fi
    if [[ -z "$IP" ]]; then
        return $ERR_IP
    else
        echo $IP
        return $ERR_OK
    fi
}

get_current_netmask() {
    NETMASK=`ifconfig eth0 | grep 'inet addr:' | cut -d: -f4`
    if [[ -z "$NETMASK" ]]; then
        NETMASK=`ifconfig eth0:avahi | grep 'inet addr:' | cut -d: -f4`
    fi
    if [[ -z "$NETMASK" ]]; then
        return $ERR_MASK "Failed to determine netmask!"
    else
        echo $NETMASK
        return $ERR_OK
    fi
}
