Ext.namespace('dSS11', 'dSS11.config');

dSS11.AddOnsOverviewPanel = Ext.extend(Ext.Panel,
    {
        initComponent: function()
        {
            this.systemStoreLoadOptions = undefined;

            this.installedStore = new Ext.data.JsonStore(
                {
                    url: '/cgi-bin/opkgwrapper.sh?action=addon-list-installed',
                    root: 'data',
                    fields:
                    [
                        'name', 'version', 'maintainer',
                        'description', 'homepage', 'tags',
                        'hrname', 'system'
                    ],
                    sortInfo:
                    {
                        field: 'hrname',
                        direction: 'ASC'
                    }
                }
            );

            this.installedStore.on('load', function(store, records, options) {
                    Ext.each(records, function(record) {
                        record.set('system', false);
                        var params = Ext.urlDecode(record.get('tags'));
                        if (params.name !== undefined) {
                            record.set('hrname', params.name);
                        } else {
                            record.set('hrname', record.get('name'));
                        }
                        record.commit();
                    }, this);

                    this.installedStore.add(this.systemStore.getRange());
                    this.installedStore.sort('hrname', 'ASC');
                    this.mainView.refresh();
                },
                this);

            this.systemStore = new Ext.data.JsonStore(
                {
                    url: '/cgi-bin/opkgwrapper.sh?action=addon-list-system',
                    root: 'data',
                    fields:
                    [
                        'name', 'version', 'maintainer',
                        'description', 'homepage', 'tags',
                        'hrname', 'system'
                    ]
                }
            );

            this.systemStore.on('load', function(store, records, options) {
                    Ext.each(records, function(record) {
                        record.set('system', true);
                        var params = Ext.urlDecode(record.get('tags'));
                        if (params.name !== undefined) {
                            record.set('hrname', params.name);
                        } else {
                            record.set('hrname', record.get('name'));
                        }
                        record.commit();
                    }, this);
                    this.installedStore.load(this.systemStoreLoadOptions);
                },
                this);

            this.loaded = false;

            this.mainView = new Ext.DataView(
                {
                    store: this.installedStore,
                    id: 'add-on-installed-view',
                    scope: this,
                    overClass: 'addon-view-over',
                    loadingText: _("Loading list of installed apps") + "...",
                    emptyText: _("You do not have any apps installed"),
                    deferEmptyText: true,
                    cls: 'x-margined-green-text',
                    tpl: new Ext.XTemplate(
                        '<tpl for=".">',
                        '<div class="{css}" qtip="<b>{hrname}<br/>Version: ' +
                        '</b>{version}<br/><br/>' +
                        '<i>{description}</i>" style="float: left">',
                        '<div class="addon-cell-item-wrap" id="{name}">',
                        '<div class="addon-cell-item">',
                        '<img src="/add-ons/{name}/default_icon.png"/>',
                        '</div>',
                        '<div class="x-green-text">',
                        '<span>{hrname}</span>',
                        '</div>',
                        '</div>',
                        '</div>',
                        '</tpl>',
                        '<div class="x-clear"></div>'
                    ),
                    itemSelector: 'div.addon-cell-item-wrap',
                    singleSelect: true,
                    listeners: {
                        dblclick: this.forwardToAddon.createDelegate(this)
                    }
                }
            );

            Ext.apply(this,
                {
                    title: _("Use"),
                    layout: 'fit',
                    tbar: [
                        {
                            xtype: 'button',
                            text: _("Open selected"),
                            iconCls: 'icon-package-open',
                            id: 'openButton',
                            tooltip: _("Hint: double click on the app icon to open the configuration interface"),
                            handler: this.forwardToSelected.createDelegate(this),
                            disabled: true,
                            scope: this
                        },
                        '->',
                        {
                            text: _("Uninstall selected"),
                            scope: this,
                            tooltip: _("Remove selected apps from your system"),
                            iconCls: 'icon-package-delete',
                            handler: this.removeAddons.createDelegate(this),
                            disabled: true,
                            id: 'uninstallButton'
                        }
                    ],
                    items:
                    [
                        {
                            xtype: 'panel',
                            border: false,
                            autoScroll: true,
                            items:
                            [
                                this.mainView
                            ]
                        }
                    ]
                }
            );

            dSS11.AddOnsOverviewPanel.superclass.initComponent.apply(this,
                                                                     arguments);
            this.on('activate',
                function() {
                    if (this.loaded === false) {
                        if (dSS11.config.updateInProgress !== true) {
                            this.forceViewLoadingMsg();
                            this.systemStore.load();
                            this.loaded = true;
                        }
                    }
                },
                this);

            dSS11.MessageBus.on('refreshaddonsview',
                function() {
                    if (this.isVisible()) {
                        this.refreshOverview();
                    }
                },
                this);

            this.mainView.on('selectionchange',
                function(dv, selections)
                {
                    var sel = this.mainView.getSelectedRecords();
                    var sys = false;
                    if (sel.length > 0) {
                        sys = sel[0].get('system');
                    }

                    var ub = this.getTopToolbar().get('uninstallButton');

                    if (sel.length > 0) {
                        this.getTopToolbar().get('openButton').enable();
                        if (sys === false) {
                            ub.enable();
                            ub.setTooltip(_("Remove selected apps from your system"));
                        } else {
                            ub.disable();
                            ub.setTooltip(_("System apps can not be " +
                                            "uninstalled"));
                        }
                    } else {
                        this.getTopToolbar().get('openButton').disable();
                        ub.disable();
                    }
                },
                this);

        }, // initcomponent
        forceViewLoadingMsg: function() {
            if (this.mainView.loadingText) {
                this.mainView.clearSelections(false, true);
                this.mainView.getTemplateTarget().update(
                    '<div class="loading-indicator">' +
                    this.mainView.loadingText + '</div>');
                this.mainView.all.clear();
            }
        },
        forwardToAddon: function(dv, index, node, ev) {
            dSS11.StatusBar.setMessage(_("The app will open in a new browser " +
                                         "tab or window"));
            window.open('/add-ons/' + dv.getRecord(node).get('name') + '/');
        },
        forwardToSelected: function()
        {
            if (this.mainView.getSelectionCount() === 0) {
                Ext.MessageBox.alert(_("No app selected"),
                      _("Please select an app to open! You can also double click the app icon to open the configuration interface"));
                return;
            }
            var selection = this.mainView.getSelectedRecords();
            var name = selection[0].get('name');
            dSS11.StatusBar.setMessage(_("The app will open in a new browser " +
                                         "tab or window"));
            window.open('/add-ons/' + name + '/');
        },
        removeAddons: function() {
            if (this.mainView.getSelectionCount() === 0) {
                Ext.MessageBox.alert(_("No app selected"),
                      _("Please select an app that should be uninstalled!"));
                return;
            }
            Ext.Msg.show({
                title: _("Uninstall apps"),
                msg: _("Are you sure that you want to uninstall the " +
                       "selected app from your system?"),
                buttons: Ext.Msg.YESNO,
                scope: this,
                fn: function(button, text) {
                    if (button === 'yes') {
                        var selection = this.mainView.getSelectedRecords();
                        var record = selection[0];

                        var fn = dSS11.util.opkg.createDelegate(this,
                            [ 'addon-remove',
                               this.findParentByType('viewport').el,
                               this.doneRemove.createDelegate(this, [ record ]),
                               record ]);
                        fn();
                    }
                }
            });
        },
        refreshOverview: function(options) {
            this.forceViewLoadingMsg();
            this.systemStoreLoadOptions = options;
            this.systemStore.load();
        },
        doneRemove: function(record) {
            var name = record.get('hrname');

            var msg =  _("App") + " <b>" + name + "</b> " +
                       _("was removed from your system");
            dSS11.StatusBar.setMessage(msg);
            Ext.Msg.show({
                title: _("Info"),
                msg: msg,
                icon: Ext.MessageBox.INFO,
                buttons: Ext.Msg.OK
            });
            this.refreshOverview();
        }
    }
);

Ext.reg('dss11addonsoverview', dSS11.AddOnsOverviewPanel);
