Ext.namespace('dSS11', 'dSS11.config');

dSS11.MailConfigPanel = Ext.extend(Ext.form.FormPanel,
{
    initComponent: function()
    {
        this.nfoURL = '/cgi-bin/mailinfo.sh';
        this.backedUp = false;
        this.formData = {};

        this.mailset = new Ext.form.FieldSet(
            {
                title:  _("Mail Server"),
                layout: 'form',
                ref:    'mailset',
                scope: this,
                items:
                [
                    {
                        xtype: 'label',
                        cls: 'x-green-text',
                        html: _("Configuration of the server settings for " +
                                "outgoing e-mails.") + "<br>" +
                              _("The mail server can be used by " +
                                "digitalSTROM-Apps to sent e-mails.")
                    },
                    {
                        xtype: 'box',
                        height: 10
                    },
                    new Ext.form.TextField({
                         width: 250,
                         name: 'from',
                         allowBlank: true,
                         fieldLabel: _("E-Mail Address"),
                         labelStyle: 'width:140px',
                         validator: this.validateMailAddress
                    }),
                    new Ext.form.TextField({
                         width: 250,
                         name: 'host',
                         allowBlank: false,
                         fieldLabel: _("Mail Server Name"),
                         labelStyle: 'width:140px',
                         validator: this.validateHostName
                    }),

                    new Ext.form.Checkbox ({
                         defaultType: 'checkboxfield',
                         boxLabel: _("Outgoing mail server requires " +
                                     "authentication"),
                         name: 'authOnOff',
                         hideLabel: true,
                         checked: true,
                         handler: function(obj, state) {
                             var uobj = this.ownerCt.findById('id-user');
                             var pobj = this.ownerCt.findById('id-password');
                             if (state == false) {
                                 uobj.disable(); pobj.disable();
                             } else {
                                 uobj.enable(); pobj.enable();
                             }
                         }
                    }),
                    {
                        xtype: 'textfield',
                        width: 250,
                        name: 'user',
                        id: 'id-user',
                        allowBlank: true,
                        validator: this.validateUser,
                        fieldLabel: _("User"),
                        labelStyle: 'margin-left: 18px; margin-right: 21px;'
                    },
                    {
                        xtype: 'textfield',
                        width: 250,
                        inputType: 'password',
                        name: 'password',
                        id: 'id-password',
                        allowBlank: true,
                        validator: this.validateUser,
                        fieldLabel: _("Password"),
                        labelStyle: 'margin-left: 18px; margin-right: 21px;'
                    },
                    {
                        xtype: 'box',
                        height: 10
                    },
                {
                    xtype: 'checkbox',
                    defaultType: 'checkboxfield',
                    boxLabel: _('Use Secure-SMTP (Port 465)'),
                    name: 'sslOnOff',
                    id: 'id-ssl',
                    checked: false,
                    hideLabel: true,
                    handler: function(obj, state) {
                        var tlsobj = this.ownerCt.findById('id-tls');
                        if (state == false) {
                            tlsobj.enable();
                        } else {
                            tlsobj.disable();
                        }
                    }
                },
                {
                    xtype: 'checkbox',
                    defaultType: 'checkboxfield',
                    boxLabel: _('Use secure data transmission (TLS)'),
                    name: 'tlsOnOff',
                    id: 'id-tls',
                    checked: false,
                    hideLabel: true
                },
                {
                    xtype: 'checkbox',
                    defaultType: 'checkboxfield',
                    boxLabel: _('Verify server certificate'),
                    name: 'certOnOff',
                    checked: false,
                    hideLabel: true
                },
                {
                    xtype: 'panel',
                    bodyBorder: false,
                    height: 40,
                    minButtonWidth: 185,
                    buttonAlign: 'center',
                    buttons: [
                        {
                            xtype: 'button',
                            text: _("Check Connection"),
                            handler: this.checkConnection.createDelegate(this)
                        },
                        {
                            xtype: 'button',
                            text: _("Send Testmail"),
                            handler: this.checkMailtransfer.createDelegate(this)
                        }
                    ]
                }
                ] // mailset items
            }
        );


        Ext.apply(this, {
            padding: 6,
            autoScroll: true,
            url: '/cgi-bin/mailconfigwrapper.sh',
            items:
            [
                {
                    xtype: 'panel',
                    border: false,
                    width: 420,
                    minButtonWidth: 80,
                    items: [ this.mailset ],
                    buttons:
                    [
                        {
                            xtype: 'button',
                            text:  _("Apply"),
                            handler: this.applyChanges.createDelegate(this)
                        },
                        {
                            xtype: 'button',
                            text:  _("Cancel"),
                            handler: this.cancelChanges.createDelegate(this),
                            scope: this
                        }
                    ]
                }
            ] // form items
        });

        this.msgready = false;

        dSS11.MailConfigPanel.superclass.initComponent.apply(this,arguments);

        this.on('activate',
                function() {
                    this.msgready = true;
                    this.reloadData(1000, this);
                    // try to backup the currently working configuration,
                    // but ignore any errors if this fails, at this stage
                    // it's not critical
                },
                this);

        this.on('beforeshow',
                function() {
                    if (this.backedUp === false) {
                        Ext.Ajax.request({
                            url: '/cgi-bin/mailconfigwrapper.sh',
                            disableCaching: true,
                            method: 'GET',
                            scope: this,
                            params: { mode: 'backup' },
                            success: function() {
                                this.backedUp = true;
                            },
                            failure: function() {}
                        });
                    }
                },
                this);

    }, // initcomponent

    reloadData: function() {
        this.el.mask(_("Loading current mail configuration") + "...",
                      'x-mask-loading');
        this.load({
            url: this.nfoURL,
            method: "GET",
            scope: this,
            disableCaching: true,
            success: function(form, action) {
                this.el.unmask();
                try {
                    var jsonData = Ext.util.JSON.decode(
                                            action.response.responseText);

                    if (jsonData.success !== true) {
                        Ext.Msg.alert(_("Error"), jsonData.message);
                        return;
                    }
                    this.formData = jsonData.data;
                } catch (error) {
                    Ext.Msg.alert(_("Error"),
                        _("Could not load mail settings") + ": "  + error.message);
                }
            },
            failure: function(form, action) {
                this.el.unmask();
                switch (action.failureType) {
                    case Ext.form.Action.CONNECT_FAILURE:
                        Ext.Msg.alert(_("Error"),
                            _("Ajax communication failed"));
                    break;
                    default:
                        Ext.Msg.alert(_("Error"),
                            _("Could not load mail settings"));
                    break;
                }
            }
        });
    },

    validateHostName: function(name) {
        if (name.length === 0) {
            return false;
        }

        var expr = /[ *?]/;
        if (expr.exec(name) != null) {
            return false;
        }

        return true;
    },

    validateUser: function(name) {
        if (name.length === 0) {
            return false;
        }

        var expr = /[ *?]/;
        if (expr.exec(name) != null) {
            return false;
        }

        return true;
    },

    validateMailAdress: function(name) {
        if (name.length === 0) {
            return false;
        }

        var expr = /[ *?]/;
        if (expr.exec(name) != null) {
            return false;
        }

        var parts = name.split("@");
        if (parts.lengh != 2) {
            return false;
        }
        var domain = parts[1].split(".");
        if (domain.length < 2) {
            return false;
        }

        return true;
    },

    cancelChanges: function() {
        this.getForm().setValues(this.formData);
    },

    applyChanges: function() {
        if (!this.getForm().isValid()) {
            Ext.Msg.alert(_("Error"),
                _("Please correct the errors in the red marked fields."));
            return;
        }
        this.el.mask(_("Applying changes") + '...', 'x-mask-loading');

        var lastSettings = this.formData;

        this.getForm().submit({
            url: this.url,
            method: 'POST',
            scope: this,
            params: { 'mode' : 'mailset' },
            success: function(form, action) {
                this.el.unmask();
                try {
                    var jsonData = Ext.util.JSON.decode(
                                            action.response.responseText);
                    this.reloadData();
                    dSS11.StatusBar.setMessage(
                        _("Mail settings applied successfully"));
                } catch (error) {
                    Ext.Msg.alert(_("Error"),
                        _("Could not load mail settings") + ": " +
                        error.message);
                }
            },
            failure: function(form, action) {
                this.el.unmask();
                if (action.failureType === Ext.form.Action.CONNECT_FAILURE) {
                    Ext.Msg.alert(_("Error"), _("Ajax communication failed"));
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(
                                            action.response.responseText);
                    Ext.Msg.alert(_("Error"),
                        _("There was an error applying your new mail settings")+
                            ": " + jsonData.message);
                    return;

                } catch (error) {
                    Ext.Msg.alert(_("Error"),
                        _("There was an error applying your new mail settings")+
                            ": "  + error.message);
                }
            }
        });
    },

    checkConnection: function() {
        this.el.mask(_("Checking Connection") + '...', 'x-mask-loading');
        this.getForm().submit({
            url: '/cgi-bin/mailconfigwrapper.sh',
            disableCaching: true,
            method: 'POST',
            scope: this,
            params: { mode: 'testconn' },
            success: function(form, action) {
                this.el.unmask();
                try {
                    var jsonData = Ext.util.JSON.decode(
                                            action.response.responseText);
                    var jsonInfo = unescape(jsonData.info);
                    if (jsonData.code != dSS11.config.codes.ERR_OK) {
                        this.showMsg(_("Error"),
                                     _("Connection test failed") + ": " +
                                     dSS11.config.err2text(jsonData.code),
                                    jsonInfo);
                    } else {
                        this.showMsg(_("Success"),
                                     _("Connection test successful"),
                                      jsonInfo);
                    }
                } catch (error) {
                    Ext.Msg.alert(_("Error"),
                            _("Could not test mail settings") + ": " +
                            error.message);
                }
            },
            failure: function(form, action) {
                this.el.unmask();
                if (action.failureType === Ext.form.Action.CONNECT_FAILURE) {
                    Ext.Msg.alert(_("Error"), _("Ajax communication failed"));
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(
                                                action.response.responseText);
                    var jsonInfo = unescape(jsonData.info);
                    this.showMsg(_("Error"),
                                 _("Connection test failed") + ": " +
                                 dSS11.config.err2text(jsonData.code),
                                 jsonInfo);
                    return;

                } catch (error) {
                    Ext.Msg.alert(_("Error"),
                                  _("Could not test mail settings") + ": " +
                                  error.message);
                }
            }
        });
    },

    checkMailtransfer: function() {
        this.el.mask(_("Sending Mail") + '...', 'x-mask-loading');
        this.getForm().submit({
            url: '/cgi-bin/mailconfigwrapper.sh',
            disableCaching: true,
            method: 'POST',
            scope: this,
            params: { mode: 'testmail' },
            success: function(form, action) {
                this.el.unmask();
                try {
                    var jsonData = Ext.util.JSON.decode(
                                                action.response.responseText);
                    var jsonInfo = unescape(jsonData.info);
                    if (jsonData.code != dSS11.config.codes.ERR_OK) {
                        this.showMsg(_("Error"),
                                     _("Mail delivery failed") + ": " +
                                     dSS11.config.err2text(jsonData.code),
                                     jsonInfo);
                    } else {
                        this.showMsg(_("Success"),
                            _("Mail delivery successful"), jsonInfo);
                    }
                } catch (error) {
                    Ext.Msg.alert(_("Error"), _("Could not send mail") + ": " +
                                  error.message);
                }
            },
            failure: function(form, action) {
                this.el.unmask();
                if (action.failureType === Ext.form.Action.CONNECT_FAILURE) {
                    Ext.Msg.alert(_("Error"), _("Ajax communication failed"));
                    return;
                }
                try {
                    var jsonData = Ext.util.JSON.decode(action.response.responseText);
                    var jsonInfo = unescape(jsonData.info);
                    this.showMsg(_("Error"), _("Mail delivery failed") + ": " +
                            dSS11.config.err2text(jsonData.code), jsonInfo);
                    return;
                } catch (error) {
                    Ext.Msg.alert(_("Error"), _("Could not send mail") + ": " +
                                  error.message);
                }
            }
        });
    },
    showMsg: function(title, msg, details) {
        var maskEl = this.findParentByType('viewport').getEl();

        maskEl.mask();

        var errorWindow = new Ext.Window({
            title: title,
            width: 450,
            padding: 6,
            layout: 'fit',
            items:
            [
                {
                    xtype: 'label',
                    text: msg
                },
                {
                    xtype: 'box',
                    height: 10
                },
                {
                    xtype: 'panel',
                    title: _("Show details"),
                    collapsible: true,
                    collapsed: true,
                    collapseFirst: true,
                    floatable: false,
                    titleCollapse: true,
                    animCollapse: false,
                    height: 200,
                    autoScroll: true,
                    items: [
                        {
                            xtype: 'panel',
                            padding: 6,
                            bodyCfg: {
                                tag: "pre"
                            },
                            html: details
                        }
                    ],
                    listeners:
                    {
                        collapse: function(p) {
                            p.setTitle(_("Show details"));
                        },
                        expand: function(p) {
                            p.setTitle(_("Hide details"));
                        }
                    }
                }
            ],
            buttonAlign: 'center',
            buttons:
            [
                {
                    text: _("OK"),
                    handler: function() {
                        errorWindow.close();
                    }
                }
            ],
            listeners:
            {
                close: function()
                {
                    maskEl.unmask();
                }
            }
        });
        errorWindow.show(this);
    }
});

Ext.reg('dss11mailconfig', dSS11.MailConfigPanel);
