Ext.namespace('dSS11', 'dSS11.config');

dSS11.NetConfigPanel = Ext.extend(Ext.form.FormPanel,
    {
        initComponent: function() 
        {
            this.nfoURL = '/cgi-bin/netinfo.sh';

            this.backedUp = false;

            this.formData = {};

            this.ipset = new Ext.form.FieldSet(
                {
                    layout: 'form',
                    ref:    'ipset',
                    scope: this,
                    items: [
                            new Ext.form.TextField({
                                 width: 140,
                                 name: 'ip',
                                 allowBlank: false,
                                 fieldLabel: _("IP Address"),
                                 validator: dSS11.util.validateIP
                            }),
                            new Ext.form.TextField({
                                 width: 140,
                                 name: 'netmask',
                                 allowBlank: false,
                                 fieldLabel: _("Netmask"),
                                 validator: dSS11.util.validateIP
                            }),
                            new Ext.form.TextField({
                                 width: 140,
                                 name: 'gateway',
                                 allowBlank: true,
                                 fieldLabel: _("Gateway"),
                                 validator: this.validateIPBlankOK.createDelegate(this)
                            }),
                            new Ext.form.TextField({
                                 width: 140,
                                 name: 'dns1',
                                 allowBlank: true,
                                 fieldLabel: _("DNS") + "1",
                                 validator: this.validateIPBlankOK.createDelegate(this)
                            }),
                            new Ext.form.TextField({
                                 width: 140,
                                 name: 'dns2',
                                 allowBlank: true,
                                 fieldLabel: _("DNS") + "2",
                                 validator: this.validateIPBlankOK.createDelegate(this)
                            }),
                            new Ext.form.DisplayField({
                                width: 140,
                                ref: 'mac',
                                name: 'mac',
                                fieldLabel: _("MAC")
                            })
                        ] // ipset items

                }
            );

            Ext.apply(this, {
                padding: 6,
                autoScroll: true,
                ref: 'ipform',
                id: 'ipform',
                url: '/cgi-bin/netcfgwrapper.sh',
                items:
                [
                    {
                        xtype: 'label',
                        anchor: '100%',
                        cls: 'x-green-text',
                        text: _("Warning! Changing settings on this page can " +
                                "make the dSS11 unreachable. Please consult " +
                                "the manual before reconfiguring the network.")
                    },
                    {
                         xtype: 'panel',
                         height: 10,
                         bodyBorder: false
                    },

                    new Ext.CycleButton({
                        width: 392,
                        showText: true,
                        id: 'mode',
                        ref: 'mode',
                        items: [
                                    {
                                        xtype: 'menucheckitem',
                                        text: _("Automatic IP address " +
                                                "configuration using DHCP"),
                                        id: 'ipdhcp',
                                        ref: 'ipdhcp'
                                    },
                                    {
                                        xtype: 'menucheckitem',
                                        text: _("Static IP address " +
                                                "configuration"),
                                        id: 'ipstatic',
                                        ref: 'ipstatic'
                                    }
                        ],
                        changeHandler: this.netCycle,
                        scope: this
                    }),
                    {
                         xtype: 'panel',
                         height: 10,
                         bodyBorder: false
                    },
                    {
                        xtype: 'label',
                        anchor: '100%',
                        ref: 'message',
                        text: "...",
                        cls: 'x-green-text'
                    },
                    {
                        xtype: 'box',
                        height: 10
                    },
                    {
                        xtype: 'panel',
                        border: false,
                        width: 392,
                        items: [ this.ipset ]
                    },
                    {
                        xtype: 'panel',
                        bodyBorder: false,
                        width: 400,
                        buttons: {
                                minButtonWidth: 80,
                                items: [
                                {
                                    xtype: 'button',
                                    text:  _("Apply"),
                                    handler: this.applyChanges.createDelegate(this)
                                },
                                {
                                    xtype: 'button',
                                    text:  _("Cancel"),
                                    handler: this.cancelChanges,
                                    scope: this
                                }
                            ]
                        }
                    }
                ] // form items
            });

            this.msgready = false;

            dSS11.NetConfigPanel.superclass.initComponent.apply(this,arguments);

            this.on('activate',
                    function() {
                        this.msgready = true;
                        this.reloadData(1000, this);
                        // try to backup the currently working configuration, 
                        // but ignore any errors if this fails, at this stage 
                        // it's not critical
                        // basically we need it only once, the very first 
                        // time, because if noone did any network changes 
                        // yet - we will not have a valid backup
                    },
                    this);

            this.on('beforeshow',
                    function() {
                        if (this.backedUp === false) {
                            Ext.Ajax.request({
                                url: '/cgi-bin/netcfgwrapper.sh',
                                disableCaching: true,
                                method: 'GET',
                                scope: this,
                                params: { mode: 'backup' },
                                success: function() {
                                    this.backedUp = true;
                                },
                                failure: function() {}
                            });
                        }
                    },
                    this);

        }, // initcomponent

    reloadData: function() {
        this.el.mask(_("Loading current network configuration") + "...", 
                      'x-mask-loading');
        this.load({
                url: this.nfoURL,
                method: "GET",
                scope: this,
                disableCaching: true,
                success: function(form, action) {
                    this.el.unmask();
                    try {
                        if (action.response.responseText.length === 0) {
                            throw { message: "no data received." };
                        }
                        var jsonData = Ext.util.JSON.decode(
                                                action.response.responseText);

                        if (jsonData.success !== true) {
                            Ext.Msg.alert(_("Error"), jsonData.message);
                            return;
                        }

                        this.formData = jsonData.data;

                        if (jsonData.data.mode === 'static') {
                            this.mode.setActiveItem(
                                              this.mode.menu.ipstatic);
                        } else {
                            this.mode.setActiveItem(
                                              this.mode.menu.ipdhcp);
                        }
                    } catch (error) {
                        Ext.Msg.alert(_("Error"), 
                            _("Could not load network settings") + 
                                ": "  + error.message);
                    }
                },
                failure: function(form, action) {
                    this.el.unmask();
                    switch (action.failureType) {
                        case Ext.form.Action.CONNECT_FAILURE:
                            Ext.Msg.alert(_("Error"), 
                                _("Ajax communication failed"));
                        break;
                        default:
                            Ext.Msg.alert(_("Error"),
                                    _("Could not load network settings"));
                        break;
                    }
                }
        });
    },

    validateIPBlankOK: function(ip) {
        if ((ip === undefined) || (ip.length === 0)) {
            return true;
        }

        return dSS11.util.validateIP(ip);
    },

    netCycle: function(btn, item) {
        var i;
        var j;
        if (item.id === 'ipdhcp') {
            if (this.msgready === true) {
                this.message.setText(_("Your network settings will be configured automatically via DHCP."));
            }
            for (i = 0; i < this.ipset.items.items.length; i++) {
                if (this.ipset.items.items[i].name !== 'mac') {
                    this.ipset.items.items[i].disable();
                }
            }

        } else if (item.id === 'ipstatic') {
            if (this.msgready === true) {
                this.message.setText(_("Please configure your network settings manually:"));
            }
            for (j = 0; j < this.ipset.items.items.length; j++) {
                this.ipset.items.items[j].enable();
            }

         }
    },
    cancelChanges: function() {
        this.getForm().setValues(this.formData);

        if ((this.formData.mode === 'static') && (this.mode.getActiveItem() != this.mode.menu.ipstatic)) {
            this.mode.setActiveItem(this.mode.menu.ipstatic);
        } else if ((this.formData.mode === 'dhcp') && (this.mode.getActiveItem() != this.mode.menu.ipdhcp)) {
            this.mode.setActiveItem(this.mode.menu.ipdhcp);
        }
    },
    restoreSettings: function() {
        Ext.Ajax.request({
            url: '/cgi-bin/netcfgwrapper.sh',
            disableCaching: true,
            method: 'GET',
            params: { mode: 'restore' },
            scope: this,
            success: function() { this.reloadData(); },
            failure: function() {
                Ext.Msg.alert(_("Error"), _("Could not restore last working configuration!"));
                this.reloadData();
            }
        });
    },
    applyChanges: function() {
        if (!this.getForm().isValid()) {
            Ext.Msg.alert(_("Error"), _("Please correct the errors in the red marked fields."));
            return;
        }
        var maskEl = this.findParentByType('viewport').el;
        maskEl.mask(_("Applying changes") + '...', 'x-mask-loading');

        var lastSettings = this.formData;

        this.getForm().submit({
            url: this.url,
            method: 'GET',
            scope: this,
            params: { 'mode' : this.mode.getActiveItem().id },
            success: function(form, action) {
                maskEl.unmask();
                try {
                    if (action.response.responseText.length === 0) {
                        throw { message: "no data received." };
                    }

                    var jsonData = Ext.util.JSON.decode(
                                            action.response.responseText);

                    if (jsonData.code === dSS11.config.codes.ERR_REDIRECT) {
                        if (jsonData.ip.length > 0) {
                            maskEl.mask(_("Redirecting to new location at") + 
                                " http://" + jsonData.ip + "/",
                                'x-mask-loading');

                            window.location = 'http://' + jsonData.ip;
                        }
                    } else {
                        this.reloadData();
                    }
                    dSS11.StatusBar.setMessage(_("Network settings applied " +
                                                 "successfully"));
                } catch (error) {
                        Ext.Msg.alert(_("Error"), 
                            _("Could not load network settings") + 
                                ": "  + error.message);
                }
            },
            failure: function(form, action) {
                maskEl.unmask();
                if (action.failureType === Ext.form.Action.CONNECT_FAILURE) {
                    Ext.Msg.alert(_("Error"), _("Ajax communication failed"));
                    return;
                }

                try {
                    if (action.response.responseText.length === 0) {
                        throw { message: "no data received." };
                    }

                    var jsonData = Ext.util.JSON.decode(
                                            action.response.responseText);
                    if (jsonData.code == dSS11.config.codes.ERR_NETUP) {
                        Ext.Msg.show({
                            title: _("Error"),
                            msg: _("There was an error applying your " + 
                                "new network settings! Do you want to " +
                                "revert to your previous working " + 
                                "configuration?"),
                                buttons: Ext.Msg.YESNO,
                                scope: this,
                                fn: function(button, text) {
                                    if (button == 'yes') {
                                        this.formData = lastSettings;
                                        this.cancelChanges();
                                        this.restoreSettings();
                                    } else {
                                        this.reloadData();
                                    }
                                },
                                animEl: 'elId',
                                icon: Ext.MessageBox.QUESTION
                            });
                        return;
                    }

                    Ext.Msg.alert(_("Error"), jsonData.message);
                    return;

                } catch (error) {
                        Ext.Msg.alert(_("Error"), 
                            _("Could not load network settings") + 
                                ": "  + error.message);
                }
            }
        });
    },
    help: function() {
    }
});

Ext.reg('dss11netconfig', dSS11.NetConfigPanel);
