Ext.namespace('dSS11', 'dSS');

dSS11.StatusBarImpl = Ext.extend(Ext.Toolbar,
    {
        initComponent: function()
        {
            this.addEvents('refreshview', 'switchview');
            this.activeMessageId = 0;
            this.clearDelay = 10000;

            this.languageStore = new Ext.data.JsonStore({
                fields: [ 'locale', 'language' ],
                root: 'supported',
                sortInfo: { field: 'language', direction: 'ASC' }
            });

            Ext.apply(this, {
                height: 30,
                layout: 'hbox',
                layoutConfig: {
                    align : 'stretch',
                    pack  : 'start'
                },
                bubbleEvents: [ 'switchview', 'refreshview' ],
                items:
                [
                    {
                        xtype: 'tbspacer',
                        width: 2
                    },
                    {
                        xtype: 'tbtext',
                        cls: 'x-black-text x-text-align-left x-text-cut',
                        ref: 'version',
                        width: 90
                    },
                    {
                        xtype: 'tbseparator',
                        width: 12
                    },
                    {
                        xtype: 'tbtext',
                        cls: 'x-black-text x-text-align-left x-text-cut',
                        ref: 'message',
                        flex: 4
                    },
                    {
                        xtype: 'tbseparator',
                        width: 12
                    },
                    {
                        xtype: 'button',
                        ref: 'refresh',
                        iconCls: 'icon-refresh-view',
                        handler: this.refreshPressed.createDelegate(this)
                    },
                    {
                        xtype: 'tbseparator',
                        width: 8
                    },
                    {
                        xtype: 'button',
                        ref: 'viewmode',
                        enableToggle: true,
                        iconCls: 'icon-advanced-view',
                        cls: 'btn-text-left',
                        width: 145,
                        disabled: true,
                        hidden: true,
                        listeners: {
                            toggle: this.viewModeChanged.createDelegate(this)
                        }
                    },
                    {
                        xtype: 'tbseparator',
                        width: 12
                    },
                    {
                        xtype: 'panel',
                        ref: 'lang',
                        border: false,
                        bodyCfg: {
                            cls: 'common-background'
                        },
                        cls: 'field-top-margin',
                        bodyBorder: false,
                        items:
                        [
                            {
                                xtype: 'combo',
                                mode: 'local',
                                name: 'locale',
                                ref: 'selection',
                                forceSelection: true,
                                setEditable: false,
                                lazyInit: true,
                                autoSelect: false,
                                width: 90,
                                store: this.languageStore,
                                displayField: 'language',
                                valueField: 'locale',
                                typeAhead: false,
                                triggerAction: 'all',
                                listeners: {
                                    select: this.selectLanguage.createDelegate(this)
                                }
                            }
                        ]
                    }
                ]
            });

            dSS11.StatusBarImpl.superclass.initComponent.apply(this, arguments);

            // it seems status bar was initialized before langpack was loaded
            // so make sure to set translated strings
            this.on('afterrender', function() {
                    this.viewmode.setText(_("Advanced view"));
                    this.viewmode.setVisible(true);
                    this.viewmode.enable();
                    this.refresh.setText(_("Refresh view"));
                    this.refresh.setVisible(true);
                    this.refresh.enable();
                },
                this);

            // don't ask me why but this was the only way how I could get a
            // regular tooltip to show up
            //
            // qtip was rendering over the button and blocking the button click,
            // so I had to go for a regular tooltip that allows some positioning
            // configuration
            this.on('afterrender', function() {
                    this.addTip.defer(1500, this);

                    this.languageStore.loadData(dSS11.config.langSupported);

                    if (dSS11.config.dSSIntegration === true)  {
                        dSS.MessageBus.on('statusmessage',
                            this.setMessage.createDelegate(this), this);
                    }

                    if (dSS11.config.dSAIntegration === true)  {
                        dSA.MessageBus.on('statusmessage',
                            this.setMessage.createDelegate(this), this);
                    }
                },
                this);

            this.languageStore.on('load', function(store, records) {
                this.lang.selection.setValue.defer(3000, this.lang.selection,
                                                  [ dSS11.config.langCurrent ]);
            },
            this);

        },
        clearMessage: function(msgID) {
            if (this.activeMessageId !== msgID) {
                return;
            }

            this.message.el.fadeOut({
                remove: false,
                useDisplay: true,
                scope: this,
                callback: function(){
                    this.message.setText('&nbsp;');
                    this.message.el.show();
                }
            });
        },
        addTip: function() {
            var tip = new Ext.ToolTip({
                html: _("Refresh view"),
                anchor: 'top',
                draggable: false,
                target: this.refresh.el
            });
            this.refresh.setTooltip(tip);

            tip = new Ext.ToolTip({
                html: _("Switch between basic and advanced view"),
                anchor: 'top',
                maxWidth: 500,
                draggable: false,
                target: this.viewmode.el
            });
            this.viewmode.setTooltip(tip);
        },
        viewModeChanged: function(button, state) {
            // switch to advanced view
            if (state === true) {
                button.setText(_("Basic view"));
                button.setIconClass('icon-default-view');
                this.setMessage(_("Switched to advanced view"));
            } else {
                button.setText(_("Advanced view"));
                button.setIconClass('icon-advanced-view');
                this.setMessage(_("Switched to basic view"));
            }

            this.fireEvent('switchview', state);
        },
        refreshPressed: function() {
            this.fireEvent('refreshview');
        },
        setVersion: function(version) {
            this.version.setText(version);
            if (this.vTip !== undefined) {
                this.vTip.destroy();
            }

            this.vTip = new Ext.ToolTip({
                html: version,
                anchor: 'top',
                anchorToTarget: false,
                draggable: false,
                maxWidth: 380,
                title: _("Version"),
                target: this.version.getEl()
            });
        },
        setMessage: function(message) {
            this.activeMessageId++;
            this.message.setText(message);
            this.clearMessage.defer(this.clearDelay, this,
                                    [this.activeMessageId]);
        },
        showAdvancedView: function() {
            this.viewmode.toggle(true);
            this.viewmode.fireEvent('toggle', this.viewmode, true);
        },
        selectLanguage: function(combo, record, index) {
            var locale = record.get('locale');
            if (dSS11.config.language === locale) {
                return;
            }

            Ext.Ajax.request({
                url: '/cgi-bin/setlocalewrapper.sh',
                params: { locale: locale },
                disableCaching: true,
                method: 'GET',
                scope: this,
                success: function(response) {
                    try {
                        var data = Ext.util.JSON.decode(response.responseText);
                        if (data.success === true) {
                            this.findParentByType('viewport').el.mask(
                                _("Applying language settings, page needs " +
                                  "to be reloaded") + "...");
                            this.setDSSLocale(record);
                        } else {
                            Ext.Msg.alert(_("Error"),
                                          _("Failed to set language"));
                            combo.setValue(dSS11.config.language);
                            return;
                        }
                    } catch (error) {
                        this.findParentByType('viewport').el.unmask();
                        Ext.Msg.alert(_("Error"),
                                      _("Failed to set language") + ": " +
                                      error);
                        combo.setValue(dSS11.config.language);
                    }
                },
                failure: function() {
                    Ext.Msg.alert(_("Error"), _("Failed to set language"));
                    combo.setValue(dSS11.config.language);
                }
            });
        },
        // if setting the property tree value was not possible,
        // there is no point to confuse the user with another
        // error message - we can't do anything here anyway
        // the dSS11 UI will be translated after reload, but the
        // dSS proptree might be out of sync (at least till next
        // dSS restart (or crash ;)
        setDSSLanguage: function(record) {
            if (dSS11.config.dSSIntegration !== true) {
                window.location.reload();
                return;
            }

            var language = record.get('language');

            Ext.Ajax.request({
                url: 'json/property/setString',
                params: {
                    path: '/system/language/language',
                    value: language
                },
                disableCaching: true,
                method: 'GET',
                scope: this,
                success: function(response) {
                    window.location.reload();
                },
                failure: function() {
                    window.location.reload();
                }
            });
        },
        setDSSLocale: function(record) {
            if (dSS11.config.dSSIntegration !== true) {
                window.location.reload();
                return;
            }

            var locale = record.get('locale');

            Ext.Ajax.request({
                url: 'json/property/setString',
                params: {
                    path: '/system/language/locale',
                    value: locale
                },
                disableCaching: true,
                method: 'GET',
                scope: this,
                success: function(response) {
                    try {
                        var data = Ext.util.JSON.decode(response.responseText);
                        if (data.ok === true) {
                            this.setDSSLanguage(record);
                        } else {
                            window.location.reload();
                        }
                    } catch (error) {
                        window.location.reload();
                    }
                },
                failure: function() {
                    // if setting the property tree value was not possible,
                    // there is no point to confuse the user with another
                    // error message - we can't do anything here anyway
                    // the dSS11 UI will be translated after reload, but the
                    // dSS proptree might be out of sync (at least till next
                    // dSS restart (or crash ;)
                    window.location.reload();
                }
            });
        }
    }
);

dSS11.StatusBar = (function() {
    return new dSS11.StatusBarImpl();
})();

